
#ifndef _LINUX_CAMAC_H
#define _LINUX_CAMAC_H

/* 
 * The major device number and the driver name
 * The device number can be overwritten during module loading
 * with the option "major=25"
 */

#define AC_MAJOR 24
#define AC_NAME "camac"

/*
 * Maximal number of crates
 */

#define AC_MAX_CRATES 15

/*
 * The crate number for broadcasting
 */

#define AC_BROADCAST_CRATE 15

/*
 * Define the default port address for the ACCAMAC interface
 * may be overwritten in the command line with the option
 * "camac=0x320"
 */

#define AC_BASE_ADDRESS 0x300

/*
 * The amount of addresses to be allocated
 */

#define AC_BASE_EXTEND 6

/*
 * Define the ioctl numbers 
 */

#define AC_TEST_CRATE 33      /* tests the aviability of the crate */
#define AC_STATUS 34          /* read X and Q flags */
#define AC_BLOCK_MODE 35      /* Block transfer mode */
#define AC_SINGLE_MODE 36     /* 16/24 bit transfer mode, default */
#define AC_ALLOW_BROADCAST 37 /* enable broadcasting for this crate */
#define AC_DENY_BROADCAST 38  /* disable broadcasting for this crate */
#define AC_C_Z_S2 39          /* make c-z-s2 cycle */
#define AC_SET_LAM 40         /* Set wait timeout for a LAM at reading */
#define AC_NOWAIT_LAM 41      /* Dont wait for a LAM at reading */
#define AC_WAIT_LAM 42        /* wait for a LAM if timeout set */

/*
 * Do not change the following ioctl numbers, they are hard-coded
 * to the according CAMAC functions
 */

#define AC_TEST_LAM 8        /* test look-at-me flags */
#define AC_CLEAR_1 9         /* clear group 1 register */
#define AC_CLEAR_LAM 10      /* clear look-at-me flags */
#define AC_CLEAR_2 11        /* clear group 2 register */
#define AC_DISABLE 24        /* disable command */
#define AC_EXECUTE 25        /* execute command */
#define AC_ENABLE 26         /* enable command */
#define AC_TEST_STATUS 27    /* test status command */

/*
 * Define maximal values for timeout and for nr of bytes
 */

#define AC_MAX_TIMEOUT 800 /* =8sec */
#define AC_MAX_WORDS 1024 /* max number of words in the block i/o mode */

/*
 * Define help macros for the N, A, F code
 */

#define NAF(n,a,f) ((((a) & 0x0f) << 10) | (((f) & 0x1f) << 5) | ((n) & 0x1f))
#define N_NAF(naf) ((naf) & 0x1f)
#define A_NAF(naf) (((naf) >> 10) & 0x0f)
#define F_NAF(naf) (((naf) >> 5) & 0x1f)

/*
 * Start a CAMAC cycle (part of the NAF code)
 */

#define AC_START_CYCLE 0x8000

/*
 * Define macros for getting the X and Q flags
 */

#define AC_Q_FLAG(qx) ((qx) & 0x01)
#define AC_X_FLAG(qx) (((qx) >> 1) & 0x01)

#endif /* _LINUX_CAMAC_H */

