/*
 * A simple CAMAC shell
 * for use with my (ole) CAMAC driver
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/camac.h>

void printhelp(void);
int cnaf24(int crate_fd, int cn, int ca, int cf, long *cdata);
void timeout(int crate_fd, int timeout);

int main(int argc, char **argv)
{
  char s[1024]; /* for the input line */
  int crate_fd;
  int crate_nr;
  int n,a,f;
  int qx;
  long data;
  int tmp;
  int tmp_fd;
  
  printf("CAMAC shell\n");

  if (argc < 2) 
    for (crate_nr = 0; crate_nr < 15; crate_nr++) {
      sprintf(s,"/dev/camac%i",crate_nr);
      if ((crate_fd = open(s,O_RDWR)) > 0) break;
    } /* for i */
  else {
    crate_nr = atoi(argv[1]);
    sprintf(s,"/dev/camac%i",crate_nr);
    if ((crate_fd = open(s,O_RDWR)) <= 0) {
      perror(s);
      exit(1);
    }
  } /* if argc < 2 */
  
  if (crate_nr == 15) {
    printf("no crate found\n");
    exit(1);
  }

  printf("camac%i > ",crate_nr);

  while(gets(s)) {
    switch (s[0]) {
    case 'n':
      if (sscanf(s,"n=%i",&tmp) == 1)
	n = tmp;
      else
	printf("Error reading Address n\n");
      break;
    case 'a':
    if (sscanf(s,"a=%i",&tmp) == 1)
      a = tmp;
    else
      printf("Error reading Subaddress a\n");
      break;
    case 'f':
      if (sscanf(s,"f=%i",&tmp) == 1)
	f = tmp;
      else
	printf("Error reading Function f\n");
      break;
    case 'd':
      if (sscanf(s,"d=%i",&tmp) == 1)
	data = tmp;
      else
	printf("Error reading Data d\n");
      break;
    case 't':
      if (sscanf(s,"t=%i",&tmp) == 1) 
	timeout(crate_fd, (tmp+5)/10);
      else
	printf("Error reading Timeout t\n");
      break;
    case 'c':
      if (sscanf(s,"c=%i",&tmp) == 1) {
	if (tmp == crate_nr) 
	  break;
	sprintf(s,"/dev/camac%i",tmp);
	if ((tmp_fd = open(s,O_RDWR)) > 0) {
	  close(crate_fd);
	  crate_nr = tmp;
	  crate_fd = tmp_fd;
	}
	else 
	  perror(s);
      } /* if sscanf */
      else
	printf("Error reading Crate number c\n");
      break;      
    case 'e':
      qx = cnaf24(crate_fd,n,a,f,&data);
    case 'p':
      printf("n=%i a=%i f=%i d=%li %s%s\n",n,a,f,data,(AC_Q_FLAG(qx))?"q":"",
	     (AC_X_FLAG(qx))?"x":"");
      break;
    case 'q':
      exit(0);
    default:
      printhelp();
    } /* switch */
    printf("camac%i > ",crate_nr);
  } /* while 1 */

  exit(2); /* never reached */

} /* main() */

void printhelp(void)
{
  printf("Syntax:\n");
  printf("n=12 ..... set address to 12\n");
  printf("a=3 ...... set subaddress to 3\n");
  printf("f=1 ...... set function code 1\n");
  printf("t=1000 ... set timeout to 1000 msec\n");
  printf("t=0 ...... set no timeout (default)\n");
  printf("c=1 ...... set CAMAC crate\n");
  printf("p ........ print parameters\n");
  printf("e ........ execute CAMAC cycle\n");
  printf("q ........ quit\n");
}

int cnaf24(int crate_fd, int cn, int ca, int cf, long *cdata)
{
  int naf = NAF(cn, ca, cf);
  int qx;

  lseek(crate_fd, naf, SEEK_SET);
  if (cf < 16) read(crate_fd, cdata, 4);
  else write(crate_fd, cdata, 4);
  ioctl(crate_fd, AC_STATUS, &qx);
  return qx;
}

void timeout(int crate_fd, int timeout)
{
  if (timeout > 0)
    ioctl(crate_fd, AC_WAIT_LAM, &timeout);
  else
    ioctl(crate_fd, AC_NOWAIT_LAM, NULL);
}
